@@ -4,6 +4,7 @@ import datasets
 from datasets import load_dataset, Dataset
 from transformers import DefaultDataCollator
 from transformers import DataCollatorForTokenClassification
+from transformers import DataCollatorForSeq2Seq
 from transformers import PreTrainedTokenizerBase
 from evaluate import load as load_metric
 from lazy_load import lazy_func
@@ -14,23 +15,31 @@ import evaluate
 from dataclasses import dataclass
 import re
 from transformers.tokenization_utils_base import PreTrainedTokenizerBase
-from frozendict import frozendict
-
+import inspect
 load_dataset = lazy_func(datasets.load_dataset)
 
-def get_name(dataset):
+def get_dataset_name(dataset):
     try:
-        s = str(dataset.cache_files.values())
-        return re.search(r"/datasets/(.*?)/default/", s).group(1).split("___")[-1]
+        s="/".join(dataset.cache_files['train'][0]['filename'].split('/huggingface/datasets/')[-1].split('/')[:-3])
+        return s
     except:
         return ""
 
+def sample_dataset(dataset,n=10000, n_eval=1000):
+    for k in dataset:
+        n_k=(n if k=='train' else n_eval)
+        if n_k and len(dataset[k])>n_k:
+            dataset[k]=dataset[k].select(range(n_k))
+    return dataset
 
 @dataclass
 class Task:
     dataset: Dataset = None
     name: str = ""
     tokenizer: PreTrainedTokenizerBase = None
+    tokenizer_kwargs: ... = fdict(padding="max_length", max_length=256,truncation=True)
+    max_rows:int=None
+    max_rows_eval:int=None
 
     def __hash__(self):
         return hash(str(self.dataset.__dict__))
@@ -45,10 +54,12 @@ class Task:
             name = "/".join(self.dataset)
             self.dataset = load_dataset(*self.dataset)
         else:
-            name = get_name(self.dataset)
+            name = get_dataset_name(self.dataset)
 
         if not self.name:
             self.name = name
+        self.dataset=sample_dataset(self.dataset,self.max_rows,self.max_rows_eval)
+
 
     def set_tokenizer(self, tokenizer):
         self.tokenizer = tokenizer
@@ -59,9 +70,6 @@ class Classification(Task):
     task_type = "SequenceClassification"
     dataset: Dataset = None
     data_collator = DefaultDataCollator()
-    tokenizer_kwargs: ... = fdict(
-            truncation=True, padding="max_length", max_length=256
-        )
     s1: str = "sentence1"
     s2: str = "sentence2"
     y: str = "labels"
@@ -71,7 +79,12 @@ class Classification(Task):
         super().__post_init__()
         if not self.num_labels:
             target = self.dataset["train"].features[self.y]
-            self.num_labels = 1 if "float" in target.dtype else target.num_classes
+            if "float" in target.dtype:
+                self.num_labels = 1
+            elif hasattr(target,'num_classes'):
+                self.num_labels=target.num_classes
+            else:
+                self.num_labels=len(set(self.dataset['train'][self.y]))
 
     def preprocess_function(self, examples):
         inputs = (
@@ -111,7 +124,8 @@ class DataCollatorForMultipleChoice:
         ]
         flattened_features = sum(flattened_features, [])
 
-        batch = self.tokenizer.pad(flattened_features, **self.tokenizer_kwargs)
+        pad_args=inspect.signature(self.tokenizer.pad).parameters.keys()
+        batch = self.tokenizer.pad(flattened_features, **fc.project(self.tokenizer_kwargs,pad_args))
 
         # Un-flatten
         batch = {k: v.view(batch_size, num_choices, -1) for k, v in batch.items()}
@@ -124,8 +138,6 @@ class DataCollatorForMultipleChoice:
 @dataclass
 class MultipleChoice(Classification):
     task_type = "MultipleChoice"
-    tokenizer_kwargs: ... = fdict(padding="max_length", max_length=256)
-
     num_labels:int = 2
     data_collator:...= DataCollatorForMultipleChoice()
     choices: ... = tuple()
@@ -134,7 +146,9 @@ class MultipleChoice(Classification):
     def __post_init__(self):
         super().__post_init__()
         self.data_collator.tokenizer_kwargs = self.tokenizer_kwargs
-
+        choices = [x for x in self.dataset['train'].features if re.match('choice\d+',x)]
+        if choices and not self.choices:
+            self.choices=choices
     def set_tokenizer(self, tokenizer):
         self.tokenizer = self.data_collator.tokenizer= tokenizer
 
@@ -152,11 +166,11 @@ class MultipleChoice(Classification):
 
         # Tokenize
         tokenized_examples = self.tokenizer(
-            first_sentences, second_sentences, truncation=True
+            first_sentences, second_sentences, **self.tokenizer_kwargs
         )
 
         # Un-flatten
-        outputs = {
+        outputs = { 
             k: [v[i : i + num_choices] for i in range(0, len(v), num_choices)]
             for k, v in tokenized_examples.items()
         }
@@ -167,17 +181,14 @@ class MultipleChoice(Classification):
 class TokenClassification(Task):
     task_type = "TokenClassification"
     dataset: Dataset = None
-    metric = evaluate.load("seqeval")
-    tokenizer_kwargs: ... = fdict(
-            truncation=True, padding="max_length", max_length=256
-        )
+    metric:... = evaluate.load("seqeval")
 
     tokens: str = None
     y: str = None
     num_labels: int = None
 
     @staticmethod
-    def align_labels_with_tokens(labels, word_ids):
+    def _align_labels_with_tokens(labels, word_ids):
         new_labels = []
         current_word = None
         for word_id in word_ids:
@@ -220,7 +231,7 @@ class TokenClassification(Task):
         new_labels = []
         for i, labels in enumerate(all_labels):
             word_ids = tokenized_inputs.word_ids(i)
-            new_labels.append(self.align_labels_with_tokens(labels, word_ids))
+            new_labels.append(self._align_labels_with_tokens(labels, word_ids))
         tokenized_inputs["labels"] = new_labels
         outputs = tokenized_inputs
         return outputs
@@ -249,3 +260,73 @@ class TokenClassification(Task):
             "accuracy": all_metrics["overall_accuracy"],
             **meta,
         }
+
+
+@dataclass
+class Seq2SeqLM(Task):
+    task_type='Seq2SeqLM'
+    data_collator:...=DataCollatorForSeq2Seq(None)
+    s1:str=''
+    s2:str=''
+    metric:...=datasets.load_metric("bleu")
+    def set_tokenizer(self,tokenizer):
+        self.tokenizer=self.data_collator.tokenizer=tokenizer
+        self.tokenizer_kwargs=self.data_collator.tokenizer_kwargs=edict(self.tokenizer_kwargs)
+
+    def preprocess_function(self, batch):
+        source, target = batch[self.s1], batch[self.s2]
+        source_tokenized = self.tokenizer(
+            source, padding="max_length", truncation=True, max_length=self.tokenizer_kwargs.max_length
+        )
+        target_tokenized = self.tokenizer(
+            target, padding="max_length", truncation=True, max_length=self.tokenizer_kwargs.max_length
+        )
+
+        batch = {k: v for k, v in source_tokenized.items()}
+        # Ignore padding in the loss
+        batch["labels"] = [
+            [-100 if token == self.tokenizer.pad_token_id else token for token in l]
+            for l in target_tokenized["input_ids"]
+        ]
+        batch['task']=[self.index]*len(batch[fc.first(batch)])
+
+        return batch
+
+    @classmethod
+    def _explode(result,prefix=''):
+        return {f'{prefix}{k}_{a}_{b}'.replace("_mid","").replace("_fmeasure",""):round(getattr(getattr(v,b),a)*100,3)\
+                for (k,v) in result.items() for a in ['precision','recall','fmeasure'] for b in ['low','mid','high']}
+
+    @classmethod
+    def _postprocess_text(preds, labels):
+        import nltk
+        preds = [pred.strip() for pred in preds]
+        labels = [label.strip() for label in labels]
+
+        # rougeLSum expects newline after each sentence
+        preds = ["\n".join(nltk.sent_tokenize(pred)) for pred in preds]
+        labels = ["\n".join(nltk.sent_tokenize(label)) for label in labels]
+
+        return preds, labels
+        
+    def compute_metrics(self, eval_preds):
+        preds, labels = eval_preds
+        if isinstance(preds, tuple):
+            preds = preds[0]
+        decoded_preds = self.tokenizer.batch_decode(preds, skip_special_tokens=True)
+        # Replace -100 in the labels as we can't decode them.
+        labels = np.where(labels != -100, labels, self.tokenizer.pad_token_id)
+        decoded_labels = self.tokenizer.batch_decode(labels, skip_special_tokens=True)
+
+        # Some simple post-processing
+        decoded_preds, decoded_labels = self._postprocess_text(decoded_preds, decoded_labels)
+        g = decoded_preds, decoded_labels
+        result = self.metric.compute(
+            predictions=decoded_preds, references=decoded_labels, use_stemmer=True
+        )
+        # Extract a few results from ROUGE
+        #result = {key: value.mid.fmeasure * 100 for key, value in result.items()}
+        result=self._explode(result)
+        meta = {"name": self.name, "size": len(decoded_preds), "index": self.index}
+
+        return {**result,**meta}
\ No newline at end of file
